"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dummyCustomSchema = void 0;
const database_engines_1 = require("../database-engines");
exports.dummyCustomSchema = {
    settings: {
        afterAll: [],
        beforeAll: [],
        engine: database_engines_1.databaseEngines.MARIADB,
        disableTriggers: false,
        ignoredTables: [],
        tablesToFill: [],
        options: [],
        values: {}
    },
    tables: [],
};
class Customizer {
    constructor(customSchema, dbConnector, logger) {
        this.customSchema = customSchema;
        this.dbConnector = dbConnector;
        this.logger = logger;
        this.extractForeignKeys = (table) => __awaiter(this, void 0, void 0, function* () {
            const foreignKeys = yield this.dbConnector.getForeignKeys(table);
            const customTable = Object.assign({
                name: '',
                columns: [],
                maxLines: 0,
            }, this.customSchema.tables.find(t => t.name.toLowerCase() === table.name.toLowerCase()));
            for (let c = 0; c < table.columns.length; c++) {
                const column = table.columns[c];
                const match = foreignKeys.find((fk) => fk.column.toLowerCase() === column.name.toLowerCase());
                if (match) {
                    column.foreignKey = { table: match.foreignTable, column: match.foreignColumn };
                    column.options.unique = column.options.unique || match.uniqueIndex;
                }
                const customColumn = customTable.columns.find(cc => cc.name.toLowerCase() === column.name.toLowerCase());
                if (customColumn) {
                    column.options = Object.assign({}, column.options, customColumn.options);
                    if (customColumn.foreignKey)
                        column.foreignKey = customColumn.foreignKey;
                    if (customColumn.values)
                        column.values = customColumn.values;
                }
                if (column.foreignKey) {
                    table.referencedTables.push(column.foreignKey.table);
                }
            }
        });
    }
    customize(schema) {
        return __awaiter(this, void 0, void 0, function* () {
            let tables = schema.tables
                .filter((table) => {
                return !this.customSchema.settings.ignoredTables.includes(table.name)
                    && (this.customSchema.settings.tablesToFill.length === 0
                        || this.customSchema.settings.tablesToFill.includes(table.name));
            });
            tables = yield Promise.all(tables.map((table) => __awaiter(this, void 0, void 0, function* () {
                yield this.customizeTable(table);
                yield this.extractForeignKeys(table);
                return table;
            })));
            this.customSchema.tables = this.orderTablesByForeignKeys(tables);
            return this.customSchema;
        });
    }
    customizeTable(table) {
        /** @todo: Remove deprecated warning */
        let useDeprecatedLines = false;
        this.customSchema.tables.forEach((table) => {
            if (table.lines) {
                useDeprecatedLines = true;
                table.maxLines = table.lines;
            }
        });
        if (useDeprecatedLines)
            this.logger.warn('DEPRECATED: Table.lines is deprecated, please use table.maxLines instead.');
        /****************/
        const customTable = this.customSchema.tables.find(t => t.name && t.name.toLowerCase() === table.name.toLowerCase());
        if (customTable) {
            table.maxLines = customTable.maxLines;
            table.addLines = customTable.addLines;
            table.disableTriggers = customTable.disableTriggers;
            table.before = customTable.before;
            table.after = customTable.after;
        }
        table.columns.forEach((column) => {
            var _a;
            const customColumn = (_a = customTable === null || customTable === void 0 ? void 0 : customTable.columns) === null || _a === void 0 ? void 0 : _a.find(c => c.name.toLowerCase() === column.name.toLowerCase());
            if (customColumn === null || customColumn === void 0 ? void 0 : customColumn.foreignKey)
                column.foreignKey = customColumn.foreignKey;
            if (customColumn === null || customColumn === void 0 ? void 0 : customColumn.values)
                column.values = customColumn.values;
            const globalSetting = this.customSchema.settings.options.find(o => o.dataTypes.includes(column.generator));
            column.options = Object.assign({}, column.options, customColumn === null || customColumn === void 0 ? void 0 : customColumn.options, globalSetting === null || globalSetting === void 0 ? void 0 : globalSetting.options);
        });
    }
    orderTablesByForeignKeys(tables) {
        let sortedTables = [];
        const recursive = (branch) => {
            const table = branch[branch.length - 1];
            while (table.referencedTables.length > 0) {
                const tableName = table.referencedTables.pop();
                const referencedTable = tables.find((t) => {
                    return t.name === tableName;
                });
                if (referencedTable)
                    recursive([].concat(branch, referencedTable));
            }
            ;
            if (sortedTables.find((t) => t.name.toLowerCase() === table.name.toLowerCase()))
                return;
            sortedTables.push({
                name: table.name,
                disableTriggers: table.disableTriggers,
                maxLines: table.maxLines,
                addLines: table.addLines,
                columns: table.columns,
                before: table.before,
                after: table.after,
                referencedTables: []
            });
            branch.pop();
            return;
        };
        tables.forEach((table) => {
            recursive([table]);
        });
        return sortedTables;
    }
    ;
}
exports.default = Customizer;
