"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const log4js_1 = require("log4js");
const clify_1 = require("@corteks/clify");
const fs = __importStar(require("fs-extra"));
const filler_1 = require("./generation/filler");
const database_connector_builder_1 = require("./database/database-connector-builder");
const path = __importStar(require("path"));
const JSONC = __importStar(require("jsonc-parser"));
const schema_class_1 = require("./schema/schema.class");
const custom_schema_class_1 = require("./schema/custom-schema.class");
const customized_schema_class_1 = require("./schema/customized-schema.class");
const logger = log4js_1.getLogger();
logger.level = 'debug';
let Main = class Main extends clify_1.CliMainClass {
    constructor() {
        super(...arguments);
        this.uri = undefined;
        this.analyse = false;
        this.reset = false;
    }
    main() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.uri)
                throw new Error('Please provide a valid database uri');
            const dbConnectorBuilder = new database_connector_builder_1.DatabaseConnectorBuilder(this.uri);
            try {
                this.dbConnector = yield dbConnectorBuilder.build();
            }
            catch (err) {
                logger.error(err.message);
                return 1;
            }
            if (!fs.pathExistsSync('settings')) {
                fs.mkdirSync('settings');
            }
            try {
                if (this.analyse) {
                    return yield this.generateSchemaFromDB();
                }
                ;
                yield this.generateData();
            }
            catch (ex) {
                if (ex.code === 'ENOENT') {
                    logger.error('Unable to read from ./settings/schema.json. Please run with --analyse first.');
                }
                else {
                    logger.error(ex);
                }
            }
            finally {
                logger.info('Close database connection');
                yield this.dbConnector.destroy();
            }
            return 0;
        });
    }
    generateSchemaFromDB() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.dbConnector)
                throw new Error('DB connection not ready');
            const schema = yield this.dbConnector.getSchema();
            fs.writeJSONSync(path.join('settings', 'schema.json'), schema.toJSON(), { spaces: 4 });
            if (!fs.existsSync(path.join('settings', 'custom_schema.jsonc'))) {
                const customSchema = new custom_schema_class_1.CustomSchema();
                fs.writeJSONSync(path.join('settings', 'custom_schema.jsonc'), customSchema, { spaces: 4 });
            }
            return 0;
        });
    }
    generateData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.dbConnector)
                throw new Error('DB connection not ready');
            const schema = yield schema_class_1.Schema.fromJSON(fs.readJSONSync(path.join('settings', 'schema.json')));
            let customSchema = new custom_schema_class_1.CustomSchema();
            try {
                customSchema = JSONC.parse(fs.readFileSync(path.join('settings', 'custom_schema.jsonc')).toString());
            }
            catch (ex) {
                logger.warn('Unable to read ./settings/custom_schema.json, this will not take any customization into account.');
            }
            const customizedSchema = customized_schema_class_1.CustomizedSchema.create(schema, customSchema);
            this.filler = new filler_1.Filler(this.dbConnector, customizedSchema, logger);
            yield this.dbConnector.backupTriggers(customSchema.tables.filter(table => table.maxLines || table.addLines).map(table => table.name));
            yield this.filler.fillTables(this.reset);
            this.dbConnector.cleanBackupTriggers();
        });
    }
    skipTable() {
        if (!this.filler)
            return;
        logger.info('Skipping...');
        this.filler.gotoNextTable();
    }
};
__decorate([
    clify_1.CliParameter({ alias: 'db', demandOption: true, description: 'Database URI. Eg: mysql://user:password@127.0.0.1:3306/database' }),
    __metadata("design:type", Object)
], Main.prototype, "uri", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", Boolean)
], Main.prototype, "analyse", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", Boolean)
], Main.prototype, "reset", void 0);
__decorate([
    clify_1.KeyPress('n', clify_1.Modifiers.NONE, 'Skip the current table. Only works during data generation phase.'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], Main.prototype, "skipTable", null);
Main = __decorate([
    clify_1.CliMain
], Main);
